
;*************************
;* NGPC Demo Clock Code  *
;*************************

; by Jeff Frohwein
; Last edit: 00-May-28

; Compiled with Macro Assembler AS v1.42 beta [Bld 3]


        cpu     93C141          ; Select Toshiba TLCS-900/H cpu

        maxmode on

START_OF_RAM    EVAL    4000h


        include "hardware.inc"  ; Include hardware defines
        include "system.inc"    ; Include System Call defines

; *** RAM Assignments ***

        RAMDB   VBlankDone
        RAMDW   Word1           ; Not used
        RAMDD   DoubleWord1     ; Not used
        RAMDB   Byte1           ; Not used


        org     200000h

; ROM Header Info

        DB      " LICENSED BY SNK CORPORATION"

        DD      Start   ; Start location

        DW      0       ; Cart ID Code (Catalog number?)
        DB      0       ; Cart Version
        DB      10h     ; System code (Monochrome = 0x00 / Color = 0x10)
;                123456789012
        DB      "JEFFCLOCK1  " ; Cart title (12 characters)
        DD      0,0,0,0 ; Reserved

; User Interrupt Vectors

UserIntVect:
        dd      nada            ; Software Interrupt (SWI 3)
        dd      nada            ; Software Interrupt (SWI 4)
        dd      nada            ; Software Interrupt (SWI 5)
        dd      nada            ; Software Interrupt (SWI 6)
        dd      nada            ; RTC Alarm Interrupt
        dd      VBlankInt       ; Vertical Blanking Interrupt
        dd      nada            ; Interrupt from Z80
        dd      nada            ; Timer Interrupt (8 bit timer 0)
        dd      nada            ; Timer Interrupt (8 bit timer 1)
        dd      nada            ; Timer Interrupt (8 bit timer 2)
        dd      nada            ; Timer Interrupt (8 bit timer 3)
        dd      nada            ; Serial Transmission Interrupt
        dd      nada            ; Serial Reception Interrupt
        dd      nada            ; (Reserved)
        dd      nada            ; End Micro DMA Int (MicroDMA 0)
        dd      nada            ; End Micro DMA Int (MicroDMA 1)
        dd      nada            ; End Micro DMA Int (MicroDMA 2)
        dd      nada            ; End Micro DMA Int (MicroDMA 3)

nada:   reti

; *** Start of User Code ***

Start:
        calr    SYS_PATCH             ; Initialize NGP or NGPC mode

        set     ANSWERB_TIME,(rUSERA) ; Shutdown on condition of no use

; Install User Interrupt Vectors

        lda     xix,(UserIntVect)
        lda     xiy,(rSWI3)
        ld      b,18
UIVloop:
        ld      xwa,(xix+)
        ld      (xiy+),xwa
        djnz    b,UIVloop

; Clear Sprite Attribute Memory

        ld      wa,0
        ld      b,64
        ld      xix,_SPRRAM
SAMloop:
        ld      (xix+),wa
        ld      (xix+),wa
        djnz    b,SAMloop

; Clear Scroll 1 & 2 VRAM with 20h,00h,20h,00h,...

        ld      xhl,_SCR1RAM
        ldw     (xhl),20h
        ld      xde,xhl
        inc     xde
        inc     xde
        ld      bc,800h-1
        ldirw

; Clear Main RAM to 0

        xor     xbc,xbc
        ld      xbc,0

        ld      xhl,_MAINRAM
        ldb     (xhl),0
        ld      xde,xhl
        inc     xde
        ld      bc,2000h-1
        ldir

; Load system font tileset

        ld      ra3,3
        ld      rw3,VECT_SYSFONTSET
        calr    SYSTEM_CALL

; Load monochrome tileset

;        ld      xhl,TileSet
;        ld      xde,_TILERAM
;        ld      xbc,256*8
;        calr    mem_CopyMono13

; Set Window X & Y Size

        ldb     (rWSIX),SCRN_X
        ldb     (rWSIY),SCRN_Y

; Set scroll 1 palette colors

        ld      xhl,ColorSet
        ld      xde,_SCR1PAL
        ld      xbc,(ColorSetEnd - ColorSet)/2
	ldirw

; Copy "Hello World" text to middle of scroll 1 RAM

        ld      xhl,HelloWorld
        ld      xde,_SCR1RAM+(SCRN_VX_B*8) + (4*2)
        ld      xbc,HelloWorldEnd - HelloWorld
        ldir

; Enable Interrupts

        ei      0

; *** Main Loop ***

MainLoop:
        cpb     (VBlankDone),0  ; Has VBlank occured?
        jr      z,MainLoop      ; no, wait until it does

        calr    GetTime

        ldb     (VBlankDone),0

; User shutdown? (Power off pressed?)

        cp      (rUSERS),0
        jr      z,MainLoop      ; no

; Power off NGP

        ld      rw3,VECT_SHUTDOWN
        calr    SYSTEM_CALL

done:   jr      done

; *** Vertical Blank Interrupt ***

VBlankInt:

; Update Watch Dog Timer to prevent CPU reset

        ldb     (rWDCR),WD_CLR

        push    xix
        push    xiy
        push    xiz
        push    xwa
        push    xbc
        push    xde
        push    xhl

        ld      a,(rS1SOX)
        inc     a
;        ld      (rS1SOX),a

        call    CopyTimeToScreen

        pop     xhl
        pop     xde
        pop     xbc
        pop     xwa
        pop     xiz
        pop     xiy
        pop     xix

        ldb     (VBlankDone),1  ; Signal end of VBlank

        reti


CopyTimeToScreen:
        ld      xiy,928ch-8
        ld      xix,400ah
        ld      b,6
        ld      w,0
CTTSloop:
        ld      a,(xix+)
        ld      (xiy+),wa
        ld      a,(xix+)
        ld      (xiy+),wa
        ld      a,(xix+)
        ld      (xiy+),wa
        djnz    b,CTTSloop

        ret

; Get time from system

GetTime:
        ld      xhl3,4002h
        ldb     rw3,VECT_RTCGET
        calr    SYSTEM_CALL

        ld      xix,400ah
        ld      xhl,4002h  ;4005
        ld      a,(xhl+)
        calr    BCDHex2AsciiHex

        ld      (xix+),2dh

        ld      a,(xhl+)
        calr    BCDHex2AsciiHex

        ld      (xix+),2dh

        ld      a,(xhl+)
        calr    BCDHex2AsciiHex

        ld      (xix+),20h
;
        ld      a,(xhl+)
        calr    BCDHex2AsciiHex

        ld      (xix+),3ah

        ld      a,(xhl+)
        calr    BCDHex2AsciiHex

        ld      (xix+),3ah

        ld      a,(xhl+)
        calr    BCDHex2AsciiHex

        ld      (xix+),20h
        ret

; Convert A from bcd hex to Ascii decimal & store at XIX

BCDHex2AsciiHex:
        ld      xiy,H2Atable
        ld      w,a
        and     w,0fh
        and     a,0f0h
        srl     4,a
        ld      a,(xiy+a)
        ld      w,(xiy+w)
        ld      (xix+),a
        ld      (xix+),w
        ret

H2Atable:
        DB      "0123456789ABCDEF"

HelloWorld:
        DB      "H",0
        DB      "e",0
        DB      "l",0
        DB      "l",0
        DB      "o",0
        DB      " ",0
        DB      "w",0
        DB      "o",0
        DB      "r",0
        DB      "l",0
        DB      "d",0
        DB      "!",0
HelloWorldEnd:

ColorSet:
        DW      0         ; Don't Care (Transparent)
        RGBSet  255,0,0   ; Red
        RGBSet  0,255,0   ; Green
        RGBSet  255,255,0 ; Yellow
ColorSetEnd:

;TileSet:
;        include "ibmpc1.inc"
;        chr_IBMPC1

SYSTEM_CALL:
        SystemCallCode
SYS_PATCH:
        SysPatchCode

	include	"fontcpy1.inc"

; Make ROM 4mbits in length to satisfy NGPC flash cart software

        org     27ffffh
        DB      0ffh

        END
